import functools
import json
from calendar import _nextmonth, _prevmonth
from datetime import datetime, timedelta

from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import ValidationError
from django.db.models import BooleanField, Case, Max, Min, Prefetch, When
from django.forms import DateField
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse, reverse_lazy
from django.utils import timezone
from django.utils.formats import date_format
from django.utils.safestring import mark_safe
from django.utils.timezone import get_current_timezone, make_aware
from django.utils.translation import gettext as _
from django.views import View
from django.views.generic import (
    CreateView,
    DeleteView,
    DetailView,
    FormView,
    ListView,
    RedirectView,
    TemplateView,
    UpdateView,
)
from django.views.generic.detail import SingleObjectMixin
from guardian.shortcuts import assign_perm, get_objects_for_user, get_users_with_perms
from recurrence.forms import RecurrenceField

from ephios.core.calendar import ShiftCalendar
from ephios.core.forms.events import EventDuplicationForm, EventForm, EventNotificationForm
from ephios.core.models import Event, EventType, Shift
from ephios.core.services.notifications.types import (
    CustomEventParticipantNotification,
    EventReminderNotification,
    NewEventNotification,
)
from ephios.core.signals import event_forms
from ephios.extra.mixins import CanonicalSlugDetailMixin, CustomPermissionRequiredMixin
from ephios.extra.permissions import get_groups_with_perms


def current_event_list_view(request):
    if request.session.get("event_list_view_type", "list") == "calendar":
        return EventCalendarView.as_view()(request)
    return EventListView.as_view()(request)


class EventListView(LoginRequiredMixin, ListView):
    model = Event

    def get_queryset(self):
        return (
            get_objects_for_user(self.request.user, "core.view_event")
            .annotate(
                start_time=Min("shifts__start_time"),
                end_time=Max("shifts__end_time"),
            )
            .annotate(
                can_change=Case(
                    When(
                        id__in=get_objects_for_user(self.request.user, ["core.change_event"]),
                        then=True,
                    ),
                    default=False,
                    output_field=BooleanField(),
                )
            )
            .filter(end_time__gte=timezone.now())
            .select_related("type")
            .prefetch_related("shifts")
            .prefetch_related(Prefetch("shifts__participations"))
            .order_by("start_time")
        )

    def get_context_data(self, *, object_list=None, **kwargs):
        kwargs.setdefault("eventtypes", EventType.objects.all())
        return super().get_context_data(**kwargs)


class EventDetailView(CustomPermissionRequiredMixin, CanonicalSlugDetailMixin, DetailView):
    model = Event
    permission_required = "core.view_event"

    def get_queryset(self):
        base = Event.objects.all()
        if self.request.user.has_perm("core.add_event"):
            base = Event.all_objects.all()
        return base.prefetch_related("shifts", "shifts__participations")


class EventEditMixin:
    @functools.cached_property
    def plugin_forms(self):
        forms = []
        for __, resp in event_forms.send(sender=None, event=self.object, request=self.request):
            forms.extend(resp)
        return forms

    def get_context_data(self, **kwargs):
        kwargs["plugin_forms"] = self.plugin_forms
        return super().get_context_data(**kwargs)

    def is_valid(self, form):
        return form.is_valid() and all(plugin_form.is_valid() for plugin_form in self.plugin_forms)

    def form_valid(self, form):
        response = super().form_valid(form)
        for plugin_form in self.plugin_forms:
            plugin_form.save()
        return response


class EventUpdateView(CustomPermissionRequiredMixin, EventEditMixin, UpdateView):
    model = Event
    queryset = Event.all_objects.all()
    permission_required = "core.change_event"

    def get_form(self, form_class=None):
        return EventForm(
            data=self.request.POST or None, user=self.request.user, instance=self.object
        )

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        form = self.get_form()
        if self.is_valid(form):
            return self.form_valid(form)
        return self.form_invalid(form)


class EventCreateView(CustomPermissionRequiredMixin, EventEditMixin, CreateView):
    template_name = "core/event_form.html"
    permission_required = "core.add_event"
    accept_object_perms = False
    model = EventType

    def post(self, request, *args, **kwargs):
        form = self.get_form()
        self.object = form.instance
        if self.is_valid(form):
            return self.form_valid(form)
        return self.form_invalid(form)

    def dispatch(self, request, *args, **kwargs):
        self.eventtype = get_object_or_404(EventType, pk=self.kwargs.get("type"))
        return super().dispatch(request, *args, **kwargs)

    def get_form(self, form_class=None):
        return EventForm(
            data=self.request.POST or None,
            user=self.request.user,
            eventtype=self.eventtype,
        )

    def get_context_data(self, **kwargs):
        inactive_events = Event.all_objects.filter(active=False)
        kwargs.setdefault("inactive_events", inactive_events)
        kwargs.setdefault("eventtype", self.eventtype)
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return reverse("core:event_createshift", kwargs={"pk": self.object.pk})


class EventActivateView(CustomPermissionRequiredMixin, SingleObjectMixin, View):
    permission_required = "core.add_event"
    queryset = Event.all_objects.all()

    def post(self, request, *args, **kwargs):
        event = self.get_object()
        try:
            event.activate()
            messages.success(
                request,
                _("The event {title} has been saved.").format(title=event.title),
            )
        except ValidationError as e:
            messages.error(request, e)
        return redirect(event.get_absolute_url())


class EventDeleteView(CustomPermissionRequiredMixin, DeleteView):
    queryset = Event.all_objects.all()
    permission_required = "core.delete_event"
    success_url = reverse_lazy("core:event_list")


class EventArchiveView(CustomPermissionRequiredMixin, ListView):
    permission_required = "core.view_past_event"
    model = Event
    template_name = "core/event_archive.html"

    def get_queryset(self):
        return (
            get_objects_for_user(self.request.user, "core.view_event")
            .annotate(
                start_time=Min("shifts__start_time"),
                end_time=Max("shifts__end_time"),
            )
            .filter(end_time__lt=timezone.now())
            .select_related("type")
            .order_by("-start_time")
        )


class EventCopyView(CustomPermissionRequiredMixin, SingleObjectMixin, FormView):
    permission_required = "core.add_event"
    model = Event
    template_name = "core/event_copy.html"
    form_class = EventDuplicationForm

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.object = self.get_object()

    def form_valid(self, form):
        occurrences = form.cleaned_data["recurrence"].between(
            datetime.now() - timedelta(days=1),
            datetime.now() + timedelta(days=730),  # allow dates up to two years in the future
            inc=True,
            dtstart=datetime.combine(
                DateField().to_python(self.request.POST["start_date"]), datetime.min.time()
            ),
        )
        for date in occurrences:
            event = self.get_object()
            start_date = event.get_start_time().date()
            shifts = list(event.shifts.all())
            event.pk = None
            event.save()
            assign_perm(
                "view_event", get_groups_with_perms(self.get_object(), ["view_event"]), event
            )
            assign_perm(
                "change_event", get_groups_with_perms(self.get_object(), ["change_event"]), event
            )
            assign_perm(
                "change_event",
                get_users_with_perms(
                    self.get_object(), only_with_perms_in=["change_event"], with_group_users=False
                ),
                event,
            )

            shifts_to_create = []
            for shift in shifts:
                shift.pk = None
                # shifts on following days should have the same offset from the new date
                offset = shift.start_time.date() - start_date
                # shifts ending on the next day should end on the next day to the new date
                end_offset = shift.end_time.date() - shift.start_time.date()
                current_tz = get_current_timezone()
                shift.end_time = make_aware(
                    datetime.combine(
                        date.date() + offset + end_offset,
                        shift.end_time.astimezone(current_tz).time(),
                    )
                )
                shift.meeting_time = make_aware(
                    datetime.combine(
                        date.date() + offset, shift.meeting_time.astimezone(current_tz).time()
                    )
                )
                shift.start_time = make_aware(
                    datetime.combine(
                        date.date() + offset, shift.start_time.astimezone(current_tz).time()
                    )
                )
                shift.event = event
                shifts_to_create.append(shift)
            Shift.objects.bulk_create(shifts_to_create)

        messages.success(self.request, _("Event copied successfully."))
        return redirect(reverse("core:event_list"))


class RRuleOccurrenceView(CustomPermissionRequiredMixin, View):
    permission_required = "core.add_event"

    def post(self, *args, **kwargs):
        try:
            recurrence = RecurrenceField().clean(self.request.POST["recurrence_string"])
            return HttpResponse(
                json.dumps(
                    recurrence.between(
                        datetime.now() - timedelta(days=1),
                        datetime.now()
                        + timedelta(days=730),  # allow dates up to two years in the future
                        inc=True,
                        dtstart=datetime.combine(
                            DateField().to_python(self.request.POST["dtstart"]), datetime.min.time()
                        ),
                    ),
                    default=lambda obj: date_format(obj, format="SHORT_DATE_FORMAT"),
                )
            )
        except (TypeError, KeyError, ValidationError):
            return HttpResponse()


class HomeView(LoginRequiredMixin, TemplateView):
    template_name = "core/home.html"


class EventNotificationView(CustomPermissionRequiredMixin, SingleObjectMixin, FormView):
    model = Event
    permission_required = "core.change_event"
    template_name = "core/event_notification.html"
    form_class = EventNotificationForm

    def get_form_kwargs(self):
        return {**super().get_form_kwargs(), "event": self.object}

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.object = self.get_object()

    def form_valid(self, form):
        action = form.cleaned_data["action"]
        if action == form.NEW_EVENT:
            NewEventNotification.send(self.object)
        elif action == form.REMINDER:
            EventReminderNotification.send(self.object)
        elif action == form.PARTICIPANTS:
            CustomEventParticipantNotification.send(self.object, form.cleaned_data["mail_content"])
        messages.success(self.request, _("Notifications sent succesfully."))
        return redirect(self.object.get_absolute_url())


class EventCalendarView(TemplateView):
    template_name = "core/event_calendar.html"

    def get_context_data(self, **kwargs):
        today = datetime.today()
        year = int(self.request.GET.get("year", today.year))
        month = int(self.request.GET.get("month", today.month))
        events = get_objects_for_user(self.request.user, "core.view_event", klass=Event)
        shifts = Shift.objects.filter(
            event__in=events, start_time__month=month, start_time__year=year
        )
        calendar = ShiftCalendar(shifts)
        kwargs.setdefault("calendar", mark_safe(calendar.formatmonth(year, month)))
        nextyear, nextmonth = _nextmonth(year, month)
        kwargs.setdefault(
            "next_month_url", f"{reverse('core:event_list')}?year={nextyear}&month={nextmonth}"
        )
        prevyear, prevmonth = _prevmonth(year, month)
        kwargs.setdefault(
            "previous_month_url", f"{reverse('core:event_list')}?year={prevyear}&month={prevmonth}"
        )
        return super().get_context_data(**kwargs)


class EventListTypeSettingView(RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        return reverse("core:event_list")

    def post(self, request, *args, **kwargs):
        request.session["event_list_view_type"] = request.POST.get("event_list_view_type")
        return self.get(request, *args, **kwargs)



# Generated by Django 3.1.4 on 2021-01-09 23:50

import datetime

from django.db import migrations, models

import ephios.extra.json


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0006_auto_20210109_2230"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="workinghours",
            name="datetime",
        ),
        migrations.AddField(
            model_name="workinghours",
            name="date",
            field=models.DateField(default=datetime.date(2021, 1, 9)),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="consequence",
            name="data",
            field=models.JSONField(
                decoder=ephios.extra.json.CustomJSONDecoder,
                default=dict,
                encoder=ephios.extra.json.CustomJSONEncoder,
            ),
        ),
        migrations.AlterField(
            model_name="workinghours",
            name="reason",
            field=models.CharField(default="", max_length=1024),
        ),
    ]



import json

from django import forms
from dynamic_preferences.types import BasePreferenceType, BaseSerializer

from ephios.extra.json import CustomJSONDecoder, CustomJSONEncoder


class JSONSerializer(BaseSerializer):
    @classmethod
    def clean_to_db_value(cls, value):
        return json.dumps(value, cls=CustomJSONEncoder, ensure_ascii=False)

    @classmethod
    def to_python(cls, value, **kwargs):
        return json.loads(value, cls=CustomJSONDecoder)


class JSONPreference(BasePreferenceType):
    serializer = JSONSerializer
    field_class = forms.CharField
    widget = forms.Textarea



from django import template
from django.db.models import Count, Q
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry
from guardian.shortcuts import get_objects_for_user

from ephios.core.consequences import editable_consequences, pending_consequences
from ephios.core.models import AbstractParticipation, Shift
from ephios.core.signup.methods import get_conflicting_participations

register = template.Library()


@register.filter(name="editable_consequences")
def editable_consequences_tag(user, states=None):
    qs = editable_consequences(user)
    if states:
        qs = qs.filter(state__in=states.split(" "))
    return qs


register.filter(name="pending_consequences", filter_func=pending_consequences)


@register.filter(name="workhour_items")
def workhour_items(user):
    return user.get_workhour_items()


@register.filter(name="qualifications_for_category")
def render_qualifications_for_category(userprofile, category_id):
    return ", ".join(
        map(
            lambda grant: grant.qualification.abbreviation,
            getattr(userprofile, f"qualifications_for_category_{category_id}"),
        )
    )


@register.filter(name="get_relevant_qualifications")
def get_relevant_qualifications(qualification_queryset):
    global_preferences = global_preferences_registry.manager()
    qs = qualification_queryset.filter(
        category__in=global_preferences["general__relevant_qualification_categories"]
    ).order_by("category", "abbreviation")
    return qs.values_list("abbreviation", flat=True)


@register.filter(name="conflicting_participations")
def participation_conflicts(participation):
    return get_conflicting_participations(
        participant=participation.participant,
        shift=participation.shift,
        start_time=participation.start_time,
        end_time=participation.end_time,
    ).values_list("shift__event__title", flat=True)


@register.filter(name="shifts_needing_disposition")
def shifts_needing_disposition(user):
    return (
        Shift.objects.filter(
            event__in=get_objects_for_user(
                user,
                perms=["core.change_event"],
            ),
            end_time__gt=timezone.now(),
        )
        .annotate(
            request_count=Count(
                "participations",
                filter=Q(participations__state=AbstractParticipation.States.REQUESTED),
            )
        )
        .filter(request_count__gt=0)
    )



import typing

from django.contrib.messages.views import SuccessMessageMixin
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic import FormView
from dynamic_preferences.forms import global_preference_form_builder

from ephios.core.signals import administration_settings_section
from ephios.extra.mixins import StaffRequiredMixin


def get_available_administration_settings_sections(request):
    sections = [
        {
            "label": _("General"),
            "url": reverse("core:settings_general"),
            "active": request.resolver_match.url_name == "settings_general",
        },
        {
            "label": _("Event types"),
            "url": reverse("core:settings_eventtype_list"),
            "active": request.resolver_match.url_name.startswith("settings_eventtype"),
        },
    ]
    for __, result in administration_settings_section.send(None, request=request):
        sections += result
    return sections


class SettingsViewMixin(FormView if typing.TYPE_CHECKING else object):
    def get_context_data(self, **kwargs):
        kwargs["settings_sections"] = get_available_administration_settings_sections(self.request)
        return super().get_context_data(**kwargs)


class GeneralSettingsView(StaffRequiredMixin, SuccessMessageMixin, SettingsViewMixin, FormView):
    template_name = "core/settings/general.html"
    success_message = _("Settings saved successfully.")

    def get_form_class(self):
        return global_preference_form_builder(section="general")

    def form_valid(self, form):
        form.update_preferences()
        return super().form_valid(form)

    def get_success_url(self):
        return reverse("core:settings_general")



# Generated by Django 3.1.6 on 2021-02-19 15:42

from django.db import migrations


def migrate_content_type(apps, schema_editor):
    from django.contrib.contenttypes.models import ContentType

    for obj in ContentType.objects.filter(app_label="user_management"):
        ContentType.objects.filter(app_label="core", model=obj.model).delete()
        obj.app_label = "core"
        obj.save()


class Migration(migrations.Migration):
    dependencies = [
        ("core", "0002_merge_content_type_20210208_0020"),
    ]

    operations = [
        migrations.RunPython(migrate_content_type),
    ]



from crispy_forms.bootstrap import FormActions, Tab, TabHolder
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Fieldset, Layout, Submit
from django import forms
from django.db.models import Exists, OuterRef
from django.forms.formsets import DELETION_FIELD_NAME
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext
from django_select2.forms import Select2MultipleWidget, Select2Widget

from ephios.core.models import Qualification, QualificationCategory, QualificationGrant, UserProfile
from ephios.extra.crispy import AbortLink
from ephios.extra.fields import EndOfDayDateTimeField
from ephios.extra.widgets import CustomDateInput
from ephios.plugins.qualification_management.importing import (
    QualificationChangeManager,
    fetch_deserialized_qualifications_from_repo,
)

# Templates in this plugin are under core/, because Qualification is a core model.


class QualificationForm(forms.ModelForm):
    class Meta:
        model = Qualification
        fields = ["title", "uuid", "abbreviation", "category", "includes"]
        widgets = {"includes": Select2MultipleWidget}
        help_texts = {
            "uuid": _(
                "Used to identify qualifications accross the ephios ecosystem. Only change if you know what you are doing."
            )
        }


class QualificationDeleteForm(forms.Form):
    fix_inclusions = forms.BooleanField(
        label=_("Maintain transitive inclusion"),
        initial=True,
        required=False,
    )
    move_grants_to_other_qualifications = forms.ModelMultipleChoiceField(
        label=_("Migrate users to other qualification"),
        required=False,
        widget=Select2MultipleWidget,
        queryset=Qualification.objects.none(),
    )

    @cached_property
    def _active_grants(self):
        return self.qualification.grants.unexpired()

    def __init__(self, *args, **kwargs):
        self.qualification = kwargs.pop("instance")
        super().__init__(*args, **kwargs)
        self.fields["fix_inclusions"].help_text = _(
            "If checked, superior qualifications ({superior}) will include inferior ({inferior}) qualifications."
        ).format(
            superior=", ".join(map(str, self.qualification.included_by.all())) or _("none"),
            inferior=", ".join(map(str, self.qualification.includes.all())) or _("none"),
        )

        self.fields["move_grants_to_other_qualifications"].queryset = Qualification.objects.exclude(
            pk=self.qualification.pk
        )
        help_text = _(
            "Select qualifications to apply to users with the qualification about to be deleted. Expiration dates will be kept."
        )

        if count := self._active_grants.count():
            help_text += " "
            help_text += ngettext(
                "There is {count} user with this qualification.",
                "There are {count} users with this qualification.",
                count,
            ).format(count=count)

        self.fields["move_grants_to_other_qualifications"].help_text = help_text

    def _move_users_to_other_qualifications(self):
        new_grants = []
        for new_qualification in self.cleaned_data["move_grants_to_other_qualifications"]:
            for old_grant in self._active_grants:
                new_grants.append(
                    QualificationGrant(
                        user_id=old_grant.user_id,
                        qualification_id=new_qualification.id,
                        expires=old_grant.expires,
                    )
                )
        QualificationGrant.objects.bulk_create(new_grants)

    def save(self):
        assert self.is_valid()
        self._move_users_to_other_qualifications()
        if self.cleaned_data["fix_inclusions"]:
            manager = QualificationChangeManager()
            manager.remove_qualifications_from_db_fixing_inclusion(self.qualification)
            manager.commit()
        else:
            self.qualification.delete()


class QualificationImportForm(forms.Form):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.deserialized_qualifications_by_uuid = {}
        self.categories_by_uuid = {}
        self.add_qualification_fields()

        self.helper = FormHelper()
        self.helper.layout = Layout(
            TabHolder(
                *[
                    Tab(category["object"].title, *category["field_names"])
                    for category in self.categories_by_uuid.values()
                ]
            ),
            FormActions(
                Submit("submit", _("Save"), css_class="float-end"),
                AbortLink(href=reverse("qualification_management:settings_qualification_list")),
            ),
        )

    def save(self):
        assert self.is_valid()
        manager = QualificationChangeManager()
        for uuid, deserialized_qualification in self.deserialized_qualifications_by_uuid.items():
            if self.cleaned_data[uuid]:
                manager.add_deserialized_qualifications_to_db(deserialized_qualification)
            else:
                manager.add_inclusions_of_deserialized_qualifications(deserialized_qualification)
        manager.commit()

    def add_qualification_fields(self):
        existing_qualification_uuids = {
            str(uuid) for uuid in Qualification.objects.all().values_list("uuid", flat=True)
        }
        for deserialized_qualification in fetch_deserialized_qualifications_from_repo():
            self.categories_by_uuid[str(deserialized_qualification.category.uuid)] = {
                "object": deserialized_qualification.category,
                "field_names": [],
            }
            self.deserialized_qualifications_by_uuid[
                str(deserialized_qualification.object.uuid)
            ] = deserialized_qualification

        for deserialized_qualification in self.deserialized_qualifications_by_uuid.values():
            uuid = str(deserialized_qualification.object.uuid)
            field = forms.BooleanField(
                required=False,
                label=deserialized_qualification.object.title,
                initial=uuid in existing_qualification_uuids,
                disabled=uuid in existing_qualification_uuids,
            )
            self.categories_by_uuid[str(deserialized_qualification.category.uuid)][
                "field_names"
            ].append(uuid)
            self.fields[uuid] = field


class BaseQualificationCategoryFormset(forms.BaseModelFormSet):
    def add_fields(self, form, index):
        super().add_fields(form, index)
        initial_form_count = self.initial_form_count()
        if self.can_delete and (self.can_delete_extra or index < initial_form_count):
            category: QualificationCategory = form.instance
            form.fields[DELETION_FIELD_NAME] = forms.BooleanField(
                label=_("Delete"),
                required=False,
                disabled=category.qualifications.exists(),
            )


QualificationCategoryFormset = forms.modelformset_factory(
    QualificationCategory,
    formset=BaseQualificationCategoryFormset,
    can_delete=True,
    extra=0,
    fields=["title", "uuid"],
)


class QualificationReassignmentForm(forms.Form):
    current_qualifications = forms.ModelMultipleChoiceField(
        label=_("current qualifications"),
        queryset=Qualification.objects.all(),
        help_text=_("Only users with all these qualifications will be selected."),
        widget=Select2MultipleWidget,
    )
    new_qualification = forms.ModelChoiceField(
        label=_("qualification to assign"),
        queryset=Qualification.objects.all(),
        widget=Select2Widget,
    )
    expires = EndOfDayDateTimeField(
        label=_("expires"),
        required=False,
        widget=CustomDateInput,
        help_text=_(
            "If empty, the new qualification will not expire. This will only be applied if a user does not already have the target qualification."
        ),
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.layout = Layout(
            Fieldset(_("User selection"), "current_qualifications"),
            Fieldset(
                _("New qualification"),
                "new_qualification",
                "expires",
            ),
            FormActions(
                Submit("submit", _("Execute"), css_class="float-end"),
                AbortLink(href=reverse("qualification_management:settings_qualification_list")),
            ),
        )

    def perform_reassignment(self):
        assert self.is_valid()
        users = UserProfile.objects.filter(
            *[
                Exists(
                    QualificationGrant.objects.unexpired().filter(
                        qualification=qualification, user=OuterRef("pk")
                    )
                )
                for qualification in self.cleaned_data["current_qualifications"]
            ]
        )
        created_count = 0
        for user in users:
            _, created = QualificationGrant.objects.get_or_create(
                user=user,
                qualification=self.cleaned_data["new_qualification"],
                defaults=dict(expires=self.cleaned_data["expires"]),
            )
            created_count += int(created)
        return created_count, len(users)



# Generated by Django 3.1.6 on 2021-02-07 12:24

import datetime
import secrets
import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import ephios.extra.json


def create_initial_permissions(apps, schema_editor):
    from django.contrib.auth.models import Group, Permission
    from django.contrib.contenttypes.models import ContentType

    group_content_type = ContentType.objects.get_for_model(Group)
    Permission.objects.create(
        name="Decide whether requested working hours should be granted",
        codename="decide_workinghours_for_group",
        content_type=group_content_type,
    )
    Permission.objects.create(
        name="Publish event for group",
        codename="publish_event_for_group",
        content_type=group_content_type,
    )


class Migration(migrations.Migration):
    replaces = [
        ("core", "0001_initial"),
        ("core", "0002_initial_permissions"),
        ("core", "0003_userprofile_calendar_token_squashed_0008_auto_20200925_1640"),
        ("core", "0004_auto_20201014_1648"),
        ("core", "0005_auto_20210106_2219"),
        ("core", "0006_auto_20210109_2230"),
        ("core", "0007_auto_20210110_0050"),
        ("core", "0008_auto_20210207_1309"),
    ]

    initial = True

    dependencies = [
        ("auth", "0011_update_proxy_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserProfile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(blank=True, null=True, verbose_name="last login"),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "email",
                    models.EmailField(max_length=254, unique=True, verbose_name="Email address"),
                ),
                ("is_active", models.BooleanField(default=True)),
                ("is_staff", models.BooleanField(default=False)),
                ("first_name", models.CharField(max_length=254, verbose_name="First name")),
                ("last_name", models.CharField(max_length=254, verbose_name="Last name")),
                ("date_of_birth", models.DateField()),
                ("phone", models.CharField(blank=True, max_length=254, null=True)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Qualification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name="QualificationGrant",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("expiration_date", models.DateField(blank=True, null=True)),
                (
                    "qualification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core.qualification",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="userprofile",
            name="calendar_token",
            field=models.CharField(default=secrets.token_urlsafe, max_length=254),
        ),
        migrations.CreateModel(
            name="QualificationCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, unique=True)),
                ("title", models.CharField(max_length=254, verbose_name="title")),
            ],
            options={
                "verbose_name": "qualification track",
                "verbose_name_plural": "qualification tracks",
            },
        ),
        migrations.RemoveField(
            model_name="qualificationgrant",
            name="expiration_date",
        ),
        migrations.AddField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(related_name="included_in_set", to="core.Qualification"),
        ),
        migrations.AddField(
            model_name="qualification",
            name="uuid",
            field=models.UUIDField(default=None, unique=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="qualificationgrant",
            name="expires",
            field=models.DateTimeField(blank=True, null=True, verbose_name="expiration date"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="grants",
                to="core.qualification",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualification_grants",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="category",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualifications",
                to="core.qualificationcategory",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="qualification",
            name="abbreviation",
            field=models.CharField(default="", max_length=254),
            preserve_default=False,
        ),
        migrations.AlterModelOptions(
            name="qualification",
            options={"verbose_name": "qualification", "verbose_name_plural": "qualifications"},
        ),
        migrations.AlterModelOptions(
            name="userprofile",
            options={"verbose_name": "user profile", "verbose_name_plural": "user profiles"},
        ),
        migrations.AlterField(
            model_name="qualification",
            name="category",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualifications",
                to="core.qualificationcategory",
                verbose_name="category",
            ),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(related_name="included_by", to="core.Qualification"),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="title",
            field=models.CharField(max_length=254, verbose_name="title"),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="core.qualification",
                verbose_name="qualification",
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="user profile",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="calendar_token",
            field=models.CharField(
                default=secrets.token_urlsafe, max_length=254, verbose_name="calendar token"
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="date_of_birth",
            field=models.DateField(verbose_name="date of birth"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="email",
            field=models.EmailField(max_length=254, unique=True, verbose_name="email address"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="first_name",
            field=models.CharField(max_length=254, verbose_name="first name"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="last_name",
            field=models.CharField(max_length=254, verbose_name="last name"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(max_length=254, null=True, verbose_name="phone number"),
        ),
        migrations.AlterField(
            model_name="qualification",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, unique=True),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="qualification",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="grants",
                to="core.qualification",
                verbose_name="qualification",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="phone",
            field=models.CharField(
                blank=True, default="", max_length=254, verbose_name="phone number"
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="qualification",
            name="included_qualifications",
            field=models.ManyToManyField(
                blank=True, related_name="included_by", to="core.Qualification"
            ),
        ),
        migrations.AlterField(
            model_name="qualificationgrant",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="qualification_grants",
                to=settings.AUTH_USER_MODEL,
                verbose_name="user profile",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="qualificationgrant",
            unique_together={("qualification", "user")},
        ),
        migrations.CreateModel(
            name="WorkingHours",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("hours", models.DecimalField(decimal_places=2, max_digits=7)),
                ("reason", models.CharField(blank=True, default="", max_length=1024)),
                ("datetime", models.DateTimeField(blank=True, null=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Consequence",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("slug", models.CharField(max_length=255)),
                ("data", models.JSONField(default=dict)),
                (
                    "state",
                    models.TextField(
                        choices=[
                            ("needs_confirmation", "needs confirmation"),
                            ("executed", "executed"),
                            ("failed", "failed"),
                            ("denied", "denied"),
                        ],
                        default="needs_confirmation",
                        max_length=31,
                    ),
                ),
                ("executed_at", models.DateTimeField(blank=True, null=True)),
                ("fail_reason", models.TextField(blank=True, max_length=255)),
                (
                    "decided_by",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="confirmed_consequences",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="confirmed by",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="affecting_consequences",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="affected user",
                    ),
                ),
            ],
        ),
        migrations.RunPython(
            code=create_initial_permissions,
        ),
        migrations.RemoveField(
            model_name="workinghours",
            name="datetime",
        ),
        migrations.AddField(
            model_name="workinghours",
            name="date",
            field=models.DateField(default=datetime.date(2021, 1, 9)),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="consequence",
            name="data",
            field=models.JSONField(
                decoder=ephios.extra.json.CustomJSONDecoder,
                default=dict,
                encoder=ephios.extra.json.CustomJSONEncoder,
            ),
        ),
        migrations.AlterField(
            model_name="workinghours",
            name="reason",
            field=models.CharField(default="", max_length=1024),
        ),
        migrations.AlterModelTable(
            name="consequence",
            table="consequence",
        ),
        migrations.AlterModelTable(
            name="qualification",
            table="qualification",
        ),
        migrations.AlterModelTable(
            name="qualificationcategory",
            table="qualificationcategory",
        ),
        migrations.AlterModelTable(
            name="qualificationgrant",
            table="qualificationgrant",
        ),
        migrations.AlterModelTable(
            name="userprofile",
            table="userprofile",
        ),
        migrations.AlterModelTable(
            name="workinghours",
            table="workinghours",
        ),
    ]



from urllib.parse import urlparse

from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import StringPreference

from ephios.core.dynamic_preferences_registry import general_global_section


@global_preferences_registry.register
class QualificationManagementReposPreference(StringPreference):
    name = "qualification_management_repos"
    verbose_name = _("List of qualification repositories")
    help_text = _("To use multiple repositories, put URLs in separate lines.")
    section = general_global_section
    default = "https://github.com/ephios-dev/ephios-qualification-fixtures/raw/main/de/_all.json"
    widget = forms.Textarea(attrs={"rows": 1})

    def validate(self, value):
        for repo_url in (line.strip() for line in value.split("\n")):
            parts = urlparse(repo_url)
            if parts.path.endswith(".json") and parts.scheme.startswith("http") and parts.netloc:
                continue
            raise ValidationError(_("'{url}' does not look like a repo url.").format(url=repo_url))



from rest_framework import serializers

from ephios.core.models import Qualification, QualificationCategory


class QualificationCategoryFixtureSerializer(serializers.ModelSerializer):
    uuid = serializers.UUIDField(label="UUID")

    class Meta:
        model = QualificationCategory
        fields = ["uuid", "title"]


class IncludedUuidsField(serializers.RelatedField):
    """This field converts a Qualification to its uuid, but keeps the UUID on deserialization."""

    def to_representation(self, value):
        return str(value.uuid)

    def to_internal_value(self, data):
        return data


class QualificationFixtureSerializer(serializers.ModelSerializer):
    uuid = serializers.UUIDField(label="UUID")
    includes = IncludedUuidsField(many=True, queryset=Qualification.objects.none())
    included_by = IncludedUuidsField(many=True, queryset=Qualification.objects.none())
    category = QualificationCategoryFixtureSerializer()

    class Meta:
        model = Qualification
        fields = [
            "uuid",
            "title",
            "abbreviation",
            "includes",
            "included_by",
            "category",
        ]



from django import forms
from django.http import Http404
from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _
from django.views import View
from django.views.generic import TemplateView
from django.views.generic.base import TemplateResponseMixin
from django.views.generic.detail import SingleObjectMixin
from django_select2.forms import ModelSelect2Widget

from ephios.core.models import (
    AbstractParticipation,
    LocalParticipation,
    Qualification,
    Shift,
    UserProfile,
)
from ephios.core.services.notifications.types import (
    ParticipationConfirmedNotification,
    ParticipationCustomizationNotification,
    ParticipationRejectedNotification,
)
from ephios.core.signup.methods import BaseParticipationForm
from ephios.extra.mixins import CustomPermissionRequiredMixin


class BaseDispositionParticipationForm(BaseParticipationForm):
    disposition_participation_template = "core/disposition/fragment_participation.html"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.can_delete = self.instance.state == AbstractParticipation.States.GETTING_DISPATCHED
        self.fields["comment"].disabled = True
        try:
            self.shift = self.instance.shift
        except AttributeError as e:
            raise ValueError(f"{type(self)} must be initialized with an instance.") from e

    class Meta(BaseParticipationForm.Meta):
        fields = ["state", "individual_start_time", "individual_end_time", "comment"]
        widgets = dict(state=forms.HiddenInput(attrs={"class": "state-input"}))


class DispositionBaseModelFormset(forms.BaseModelFormSet):
    """
    To allow us to dynamically add server-side rendered forms to a formset
    we patch a way to change the starting index.
    """

    def __init__(self, *args, start_index=0, **kwargs):
        self._start_index = start_index
        super().__init__(*args, **kwargs)

    def add_prefix(self, index):
        return f"{self.prefix}-{self._start_index + index}"

    def save_existing(self, form, instance, commit=True):
        """Existing participation state overwrites the getting dispatched state."""
        if form.instance.state == AbstractParticipation.States.GETTING_DISPATCHED:
            form.instance.state = AbstractParticipation.objects.get(id=form.instance.id).state
        return form.save(commit=commit)


def get_disposition_formset(form):
    return forms.modelformset_factory(
        model=AbstractParticipation,
        formset=DispositionBaseModelFormset,
        form=form,
        extra=0,
        can_order=False,
        can_delete=True,
    )


def addable_users(shift):
    """
    Return queryset of user objects that can be added to the shift.
    This also includes users that already have a participation, as that might have gotten removed in JS.

    This also includes users that can normally not see the event. The permission will be added accordingly.
    If needed, this method could be moved to signup methods.
    """
    return UserProfile.objects.all()


class AddUserForm(forms.Form):
    user = forms.ModelChoiceField(
        widget=ModelSelect2Widget(
            model=UserProfile,
            search_fields=["first_name__icontains", "last_name__icontains"],
            attrs={
                "form": "add-user-form",
                "data-placeholder": _("search"),
                "data-tags": "true",
                "data-token-separators": [],
            },
        ),
        queryset=UserProfile.objects.none(),  # set using __init__
    )
    new_index = forms.IntegerField(widget=forms.HiddenInput)

    def __init__(self, user_queryset, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["user"].queryset = user_queryset


class DispositionBaseViewMixin(CustomPermissionRequiredMixin, SingleObjectMixin):
    permission_required = "core.change_event"
    model = Shift

    def setup(self, request, *args, **kwargs):
        super().setup(request, *args, **kwargs)
        self.object: Shift = self.get_object()

    def dispatch(self, request, *args, **kwargs):
        if self.object.signup_method.disposition_participation_form_class is None:
            raise Http404(_("This signup method does not support disposition."))
        return super().dispatch(request, *args, **kwargs)

    def get_permission_object(self):
        return self.object.event


class AddUserView(DispositionBaseViewMixin, TemplateResponseMixin, View):
    def get_template_names(self):
        return [
            self.object.signup_method.disposition_participation_form_class.disposition_participation_template
        ]

    def post(self, request, *args, **kwargs):
        shift = self.object
        form = AddUserForm(
            data=request.POST,
            user_queryset=addable_users(shift),
        )
        if form.is_valid():
            user: UserProfile = form.cleaned_data["user"]
            instance = shift.signup_method.get_participation_for(user.as_participant())
            instance.state = AbstractParticipation.States.GETTING_DISPATCHED
            instance.save()

            DispositionParticipationFormset = get_disposition_formset(
                self.object.signup_method.disposition_participation_form_class
            )
            formset = DispositionParticipationFormset(
                queryset=AbstractParticipation.objects.filter(pk=instance.pk),
                prefix="participations",
                start_index=form.cleaned_data["new_index"],
            )
            form = next(filter(lambda form: form.instance.id == instance.id, formset))
            return self.render_to_response({"form": form, "shift": shift})
        raise Http404()


class AddPlaceholderParticipantView(DispositionBaseViewMixin, TemplateResponseMixin, View):
    def get_template_names(self):
        return [
            self.object.signup_method.disposition_participation_form_class.disposition_participation_template
        ]

    def post(self, request, *args, **kwargs):
        shift = self.object
        from ephios.core.signup.participants import PlaceholderParticipant

        participant = PlaceholderParticipant(
            first_name=request.POST["first_name"],
            last_name=request.POST["last_name"],
            qualifications=Qualification.objects.none(),
            email=None,
            date_of_birth=None,
        )
        instance = shift.signup_method.get_participation_for(participant)
        instance.state = AbstractParticipation.States.GETTING_DISPATCHED
        instance.save()

        DispositionParticipationFormset = get_disposition_formset(
            self.object.signup_method.disposition_participation_form_class
        )
        formset = DispositionParticipationFormset(
            queryset=AbstractParticipation.objects.filter(pk=instance.pk),
            prefix="participations",
            start_index=int(request.POST["new_index"]),
        )
        form = next(filter(lambda form: form.instance.id == instance.id, formset))
        return self.render_to_response({"form": form, "shift": shift})


class DispositionView(DispositionBaseViewMixin, TemplateView):
    template_name = "core/disposition/disposition.html"

    def get_formset(self):
        DispositionParticipationFormset = get_disposition_formset(
            self.object.signup_method.disposition_participation_form_class
        )
        formset = DispositionParticipationFormset(
            self.request.POST or None,
            queryset=self.object.participations.all(),
            prefix="participations",
        )
        return formset

    def _send_participant_notifications(self, formset):
        for participation, changed_fields in formset.changed_objects:
            if (
                participation.get_real_instance_class() != LocalParticipation
                or participation.user != self.request.user
            ):
                if "state" in changed_fields:
                    # send state updates
                    if participation.state == AbstractParticipation.States.CONFIRMED:
                        ParticipationConfirmedNotification.send(participation)
                    elif participation.state == AbstractParticipation.States.RESPONSIBLE_REJECTED:
                        ParticipationRejectedNotification.send(participation)
                elif participation.state == AbstractParticipation.States.CONFIRMED:
                    form: BaseParticipationForm = next(
                        filter(lambda f, p=participation: f.instance == p, formset.forms)
                    )
                    if claims := form.get_customization_notification_info():
                        # If state didn't change, but confirmed participation was customized, notify about that.
                        ParticipationCustomizationNotification.send(participation, claims)

    def post(self, request, *args, **kwargs):
        formset = self.get_formset()
        if not formset.is_valid():
            return self.get(request, *args, **kwargs, formset=formset)

        formset.save()
        self._send_participant_notifications(formset)

        # non_polymorphic() needed because of https://github.com/django-polymorphic/django-polymorphic/issues/34
        self.object.participations.filter(
            state=AbstractParticipation.States.GETTING_DISPATCHED
        ).non_polymorphic().delete()
        return redirect(self.object.event.get_absolute_url())

    def get_context_data(self, **kwargs):
        kwargs.setdefault("formset", self.get_formset())
        kwargs.setdefault("states", AbstractParticipation.States)
        kwargs.setdefault(
            "participant_template",
            self.object.signup_method.disposition_participation_form_class.disposition_participation_template,
        )
        kwargs.setdefault(
            "render_requested_state",
            self.object.signup_method.uses_requested_state
            or self.object.participations.filter(
                state=AbstractParticipation.States.REQUESTED
            ).exists(),
        )
        kwargs.setdefault(
            "add_user_form",
            AddUserForm(user_queryset=addable_users(self.object)),
        )
        return super().get_context_data(**kwargs)



from django.contrib import messages
from django.db import transaction
from django.http import HttpResponseRedirect, JsonResponse
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from django.views import View
from django.views.generic import CreateView, FormView, ListView, UpdateView

from ephios.core.models import Qualification, QualificationCategory
from ephios.core.views.settings import SettingsViewMixin
from ephios.extra.mixins import StaffRequiredMixin
from ephios.plugins.qualification_management.forms import (
    QualificationCategoryFormset,
    QualificationDeleteForm,
    QualificationForm,
    QualificationImportForm,
    QualificationReassignmentForm,
)
from ephios.plugins.qualification_management.importing import RepoError
from ephios.plugins.qualification_management.serializers import QualificationFixtureSerializer

# Templates in this plugin are under core/, because Qualification is a core model.


class QualificationListView(StaffRequiredMixin, SettingsViewMixin, ListView):
    model = Qualification
    ordering = ("category__title", "title")


class QualificationImportView(StaffRequiredMixin, SettingsViewMixin, FormView):
    template_name = "core/import.html"
    form_class = QualificationImportForm

    def get_success_url(self):
        return reverse("qualification_management:settings_qualification_list")

    def dispatch(self, request, *args, **kwargs):
        try:
            return super().dispatch(request, *args, **kwargs)
        except RepoError:
            messages.error(
                request,
                _(
                    "There was an error fetching one of the qualification repos. "
                    "Check the URLs, try again or contact support."
                ),
            )
            return HttpResponseRedirect(self.get_success_url())

    def form_valid(self, form):
        form.save()
        return super().form_valid(form)


class QualificationCreateView(StaffRequiredMixin, SettingsViewMixin, CreateView):
    model = Qualification
    form_class = QualificationForm

    def get_form_kwargs(self):
        return {
            **super().get_form_kwargs(),
            "instance": Qualification(is_imported=False),
        }

    def get_success_url(self):
        messages.success(self.request, _("Qualification was saved."))
        return reverse("qualification_management:settings_qualification_list")


class QualificationUpdateView(StaffRequiredMixin, SettingsViewMixin, UpdateView):
    model = Qualification
    form_class = QualificationForm

    def get_success_url(self):
        messages.success(self.request, _("Qualification was saved."))
        return reverse("qualification_management:settings_qualification_list")


class QualificationDeleteView(StaffRequiredMixin, SettingsViewMixin, UpdateView):
    model = Qualification
    form_class = QualificationDeleteForm
    template_name_suffix = "_confirm_delete"

    def get_success_url(self):
        messages.info(self.request, _("Qualification was deleted."))
        return reverse("qualification_management:settings_qualification_list")


class QualificationCategorySetUpdateView(StaffRequiredMixin, SettingsViewMixin, FormView):
    form_class = QualificationCategoryFormset
    template_name = "core/qualificationcategories_form.html"

    def get_form_kwargs(self):
        return {"queryset": QualificationCategory.objects.all(), **super().get_form_kwargs()}

    def form_valid(self, form):
        with transaction.atomic():
            form.save()
        return super().form_valid(form)

    def get_success_url(self):
        messages.info(self.request, _("Qualification categories saved."))
        return reverse("qualification_management:settings_qualification_list")


class QualificationReassignmentView(StaffRequiredMixin, SettingsViewMixin, FormView):
    form_class = QualificationReassignmentForm
    template_name = "core/qualification_reassignment.html"

    def form_valid(self, form):
        with transaction.atomic():
            created, _ = form.perform_reassignment()
        messages.info(
            self.request,
            ngettext_lazy(
                "One user had '{qualification}' assigned.",
                "{count} users had '{qualification}' assigned.",
                created,
            ).format(count=str(created), qualification=str(form.cleaned_data["new_qualification"])),
        )
        return super().form_valid(form)

    def get_success_url(self):
        return reverse("qualification_management:settings_qualification_list")


class QualificationExportFixtureView(StaffRequiredMixin, View):
    def get(self, request, *args, **kwargs):
        qualifications = Qualification.objects.all()
        serializer = QualificationFixtureSerializer(qualifications, many=True)
        return JsonResponse(serializer.data, safe=False, json_dumps_params=dict(ensure_ascii=False))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ephios',
 'ephios.core',
 'ephios.core.forms',
 'ephios.core.migrations',
 'ephios.core.models',
 'ephios.core.services',
 'ephios.core.services.notifications',
 'ephios.core.signup',
 'ephios.core.templatetags',
 'ephios.core.views',
 'ephios.extra',
 'ephios.extra.management',
 'ephios.extra.management.commands',
 'ephios.extra.templatetags',
 'ephios.modellogging',
 'ephios.modellogging.migrations',
 'ephios.modellogging.templatetags',
 'ephios.plugins',
 'ephios.plugins.basesignup',
 'ephios.plugins.basesignup.signup',
 'ephios.plugins.eventautoqualification',
 'ephios.plugins.eventautoqualification.migrations',
 'ephios.plugins.guests',
 'ephios.plugins.guests.migrations',
 'ephios.plugins.pages',
 'ephios.plugins.pages.migrations',
 'ephios.plugins.qualification_management',
 'ephios.plugins.qualification_management.migrations']

package_data = \
{'': ['*'],
 'ephios': ['locale/de/LC_MESSAGES/django.po',
            'static/bootstrap/*',
            'static/bootstrap/js/*',
            'static/bootstrap/scss/*',
            'static/bootstrap/scss/forms/*',
            'static/bootstrap/scss/helpers/*',
            'static/bootstrap/scss/mixins/*',
            'static/bootstrap/scss/utilities/*',
            'static/bootstrap/scss/vendor/*',
            'static/clipboardjs/js/*',
            'static/ephios/img/*',
            'static/ephios/js/*',
            'static/ephios/js/formset/*',
            'static/ephios/scss/*',
            'static/fontawesome/css/*',
            'static/fontawesome/webfonts/*',
            'static/jquery/js/*',
            'static/plugins/basesignup/js/*',
            'static/select2/css/*',
            'static/select2/js/*',
            'static/select2/js/i18n/*',
            'static/sortablejs/*',
            'templates/*',
            'templates/registration/*'],
 'ephios.core': ['templates/core/*',
                 'templates/core/disposition/*',
                 'templates/core/fragments/*',
                 'templates/core/mails/*',
                 'templates/core/settings/*'],
 'ephios.extra': ['templates/extra/widgets/*'],
 'ephios.modellogging': ['templates/modellogging/*'],
 'ephios.plugins': ['basesignup/templates/basesignup/*',
                    'basesignup/templates/basesignup/section_based/*',
                    'guests/templates/guests/*',
                    'pages/templates/pages/*',
                    'qualification_management/templates/core/*']}

install_requires = \
['bleach>=3.2.1,<5.0.0',
 'crispy-bootstrap5>=0.4,<0.7',
 'django-compressor>=2.4,<3.0',
 'django-crispy-forms>=1.11.1,<2.0.0',
 'django-csp>=3.7,<4.0',
 'django-dynamic-preferences>=1.11.0,<2.0.0',
 'django-environ>=0.4.5,<0.9.0',
 'django-formset-js-improved>=0.5.0,<0.6.0',
 'django-guardian>=2.3.0,<3.0.0',
 'django-ical>=1.7.1,<2.0.0',
 'django-libsass>=0.8,<0.10',
 'django-polymorphic>=3.0.0,<4.0.0',
 'django-select2>=7.4.2,<8.0.0',
 'django-statici18n>=2.0.1,<3.0.0',
 'django-webpush>=0.3.3,<0.4.0',
 'django>=3.2,<3.3',
 'djangorestframework>=3.12.4,<4.0.0',
 'markdown>=3.2.2,<4.0.0',
 'reportlab>=3.5.51,<4.0.0']

extras_require = \
{':python_version < "3.9"': ['importlib-metadata>=3.7.3,<5.0.0'],
 'mysql': ['mysqlclient>=2.0.1,<3.0.0'],
 'pgsql': ['psycopg2>=2.8.6,<3.0.0'],
 'redis': ['django-redis>=4.12.1,<6.0.0']}

setup_kwargs = {
    'name': 'ephios',
    'version': '0.10.2',
    'description': 'ephios is a tool to manage shifts for medical services.',
    'long_description': '![tests](https://github.com/ephios-dev/ephios/workflows/tests/badge.svg)\n[![Documentation Status](https://readthedocs.org/projects/ephios/badge/?version=latest)](https://docs.ephios.de/en/latest/?badge=latest)\n[![PyPI](https://img.shields.io/pypi/v/ephios)](https://pypi.org/project/ephios/)\n[![Coverage Status](https://coveralls.io/repos/github/ephios-dev/ephios/badge.svg?branch=main)](https://coveralls.io/github/ephios-dev/ephios?branch=main)\n[![translated by Weblate](https://hosted.weblate.org/widgets/ephios/-/svg-badge.svg)](https://hosted.weblate.org/engage/ephios/)\n\n\n# ephios\n\nephios is a tool to manage shifts for medical services.\nPlanners can create events for which volunteer help is required (e.g. security/medical services, beach patrols, exercises).\nAn event can contain multiple shifts for which different processes can be applied for signup \n(e.g. a direct confirmation for an event or an "application" that has to be accepted first).\nThe volunteers can register for the respective shifts via a clearly arranged web interface.\nThe planners can then assign personnel and have an overview of the current status.\nAround this central feature there are further supporting functions like the management of the volunteers and their\nqualifications or an overview of the volunteer hours worked. A flexible group systems helps to support complex permission\nszenarios. The functionality can be extended using plugins.\n\n## Documentation\nYou can find the documentation for ephios at [Read the Docs](https://docs.ephios.de/en/latest). This includes\nthe user guide and installation instructions.\n\n## Contributing\nContributions to ephios are very welcome. You can find information about contributing at our [Contribution page](https://docs.ephios.de/en/latest/development/contributing.html)\nWe are using Weblate for translations, you can also contribute [there](https://hosted.weblate.org/engage/ephios/).\n',
    'author': 'Julian Baumann',
    'author_email': 'julian@ephios.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://ephios.de',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import dataclasses
import functools
from datetime import date
from typing import Optional

from django.contrib.auth import get_user_model
from django.db.models import QuerySet
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _

from ephios.core.models import AbstractParticipation, LocalParticipation, Qualification
from ephios.core.models.events import PlaceholderParticipation


@dataclasses.dataclass(frozen=True)
class AbstractParticipant:
    first_name: str
    last_name: str
    qualifications: QuerySet = dataclasses.field(hash=False)
    date_of_birth: Optional[date]
    email: Optional[str]  # if set to None, no notifications are sent

    def get_age(self, today: date = None):
        if self.date_of_birth is None:
            return None
        today, born = today or date.today(), self.date_of_birth
        return today.year - born.year - ((today.month, today.day) < (born.month, born.day))

    def __str__(self):
        return f"{self.first_name} {self.last_name}"

    def new_participation(self, shift):
        raise NotImplementedError

    def participation_for(self, shift):
        """Return the participation object for a shift. Return None if it does not exist."""
        raise NotImplementedError

    def all_participations(self):
        """Return all participations for this participant"""
        raise NotImplementedError

    @functools.lru_cache
    def collect_all_qualifications(self) -> set:
        return Qualification.collect_all_included_qualifications(self.qualifications)

    def has_qualifications(self, qualifications):
        return set(qualifications) <= self.collect_all_qualifications()

    def reverse_signup_action(self, shift):
        raise NotImplementedError

    def reverse_event_detail(self, event):
        raise NotImplementedError

    @property
    def icon(self):
        return mark_safe('<span class="fa fa-user"></span>')


@dataclasses.dataclass(frozen=True)
class LocalUserParticipant(AbstractParticipant):
    user: get_user_model()

    def new_participation(self, shift):
        return LocalParticipation(shift=shift, user=self.user)

    def participation_for(self, shift):
        try:
            return LocalParticipation.objects.get(shift=shift, user=self.user)
        except LocalParticipation.DoesNotExist:
            return None

    def all_participations(self):
        return LocalParticipation.objects.filter(user=self.user)

    def reverse_signup_action(self, shift):
        return reverse("core:signup_action", kwargs=dict(pk=shift.pk))

    def reverse_event_detail(self, event):
        return event.get_absolute_url()


@dataclasses.dataclass(frozen=True)
class PlaceholderParticipant(AbstractParticipant):
    def new_participation(self, shift):
        return PlaceholderParticipation(
            shift=shift, first_name=self.first_name, last_name=self.last_name
        )

    def participation_for(self, shift):
        try:
            return PlaceholderParticipation.objects.get(
                shift=shift, first_name=self.first_name, last_name=self.last_name
            )
        except PlaceholderParticipation.DoesNotExist:
            return None

    def all_participations(self):
        return AbstractParticipation.objects.none()

    def reverse_signup_action(self, shift):
        raise NotImplementedError

    def reverse_event_detail(self, event):
        raise NotImplementedError

    @property
    def icon(self):
        return mark_safe(
            f'<span class="fa fa-user-tag" data-toggle="tooltip" data-placement="left" title="{_("Placeholder")}"></span>'
        )



from django import forms
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.db import IntegrityError
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import CreateView, DetailView
from django_select2.forms import Select2MultipleWidget
from dynamic_preferences.registries import global_preferences_registry

from ephios.core.models import Event
from ephios.core.views.signup import BaseShiftActionView
from ephios.extra.widgets import CustomDateInput
from ephios.plugins.guests.models import EventGuestShare, GuestUser


class RedirectAuthenticatedUserMixin:
    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated:
            return redirect(self.get_authenticated_url())
        if "ephios.plugins.guests" not in global_preferences_registry.manager().get(
            "general__enabled_plugins"
        ):
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def get_authenticated_url(self):
        return reverse("core:index")


class GuestRegistrationView(RedirectAuthenticatedUserMixin, CreateView):
    model = GuestUser
    form_class = forms.modelform_factory(
        GuestUser,
        fields=["email", "first_name", "last_name", "date_of_birth", "phone", "qualifications"],
        widgets={
            "qualifications": Select2MultipleWidget,
            "date_of_birth": CustomDateInput,
        },
    )

    def get_event(self):
        try:
            return Event.all_objects.get(pk=self.kwargs.get("event_id"))
        except Event.DoesNotExist as e:
            raise PermissionDenied() from e

    def get_authenticated_url(self):
        messages.info(self.request, _("Log out to register as guest."))
        return self.get_event().get_absolute_url()

    def get_context_data(self, **kwargs):
        kwargs.setdefault("event", self.event)
        return super().get_context_data(**kwargs)

    def dispatch(self, request, *args, **kwargs):
        self.event = self.get_event()
        input_token = kwargs["public_signup_token"]
        try:
            EventGuestShare.objects.get(event=self.event, active=True, token=input_token)
        except EventGuestShare.DoesNotExist as e:
            raise PermissionDenied from e
        return super().dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        form.instance.event = self.event
        try:
            guest = form.save()
        except IntegrityError:
            # unique_together constraint not passed
            form.add_error(
                None,
                _("You already registered as a guest for this event."),
            )
            return self.form_invalid(form)
        messages.info(self.request, _("Save the URL of this page to access this site later."))
        return redirect(guest.as_participant().reverse_event_detail(self.event))


class GuestEventDetailView(RedirectAuthenticatedUserMixin, DetailView):
    model = GuestUser
    template_name = "guests/event_detail.html"

    def get_authenticated_url(self):
        messages.info(self.request, _("Log out to view the guest version of this page."))
        return self.get_object().event.get_absolute_url()

    def get_context_data(self, **kwargs):
        kwargs["event"] = self.get_object().event
        return super().get_context_data(**kwargs)

    def get_object(self, queryset=None):
        try:
            return GuestUser.objects.get(access_token=self.kwargs.get("guest_access_token"))
        except GuestUser.DoesNotExist as e:
            raise PermissionDenied from e


class GuestUserShiftActionView(RedirectAuthenticatedUserMixin, BaseShiftActionView):
    def get_participant(self):
        try:
            return GuestUser.objects.get(
                access_token=self.kwargs.get("guest_access_token")
            ).as_participant()
        except GuestUser.DoesNotExist as e:
            raise PermissionDenied from e



from django.views import View
from django.views.generic.detail import SingleObjectMixin

from ephios.core.models import Shift
from ephios.core.signup.methods import get_nonlocal_participant_from_request
from ephios.extra.mixins import CustomPermissionRequiredMixin


def request_to_participant(request):
    if request.user.is_authenticated:
        return request.user.as_participant()
    return get_nonlocal_participant_from_request(request)


class BaseShiftActionView(SingleObjectMixin, View):
    model = Shift

    def dispatch(self, request, *args, **kwargs):
        return self.get_object().signup_method.signup_view(
            request, *args, **{**kwargs, "participant": self.get_participant()}
        )

    def get_participant(self):
        raise NotImplementedError


class LocalUserShiftActionView(CustomPermissionRequiredMixin, BaseShiftActionView):
    permission_required = "core.view_event"

    def get_permission_object(self):
        return self.get_object().event

    def get_participant(self):
        return self.request.user.as_participant()



from django.contrib.auth.models import Group, Permission
from django.forms import BooleanField
from guardian.ctypes import get_content_type
from guardian.shortcuts import assign_perm, remove_perm
from guardian.utils import get_group_obj_perms_model


def get_groups_with_perms(obj, only_with_perms_in):
    ctype = get_content_type(obj)
    group_model = get_group_obj_perms_model(obj)

    group_rel_name = group_model.group.field.related_query_name()

    if group_model.objects.is_generic():
        group_filters = {
            f"{group_rel_name}__content_type": ctype,
            f"{group_rel_name}__object_pk": obj.pk,
        }
    else:
        group_filters = {f"{group_rel_name}__content_object": obj}

    permission_ids = Permission.objects.filter(
        content_type=ctype, codename__in=only_with_perms_in
    ).values_list("id", flat=True)
    group_filters.update(
        {
            f"{group_rel_name}__permission_id__in": permission_ids,
        }
    )
    return Group.objects.filter(**group_filters).distinct()


class PermissionField(BooleanField):
    """
    This field takes a list of permissions and a permission_target and renders a checkbox that is checked if the target
    has all given permissions. It requires a permission_target attribute on the form as well as calling the appropriate
    methods which is taken care of by PermissionFormMixin
    """

    def __init__(self, *args, **kwargs):
        self.permission_set = kwargs.pop("permissions")
        super().__init__(*args, **kwargs)

    def set_initial_value(self, user_or_group):
        self.target = user_or_group
        codename_set = set(map(lambda perm: perm.split(".")[-1], self.permission_set))
        self.initial = codename_set <= set(
            self.target.permissions.values_list("codename", flat=True)
        )

    def assign_permissions(self, target):
        for permission in self.permission_set:
            assign_perm(permission, target)

    def remove_permissions(self, target):
        for permission in self.permission_set:
            remove_perm(permission, target)


class PermissionFormMixin:
    """
    Mixin for django.forms.ModelForm that handles permission updates for all ephios.extra.permissions.PermissionField on that form
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field in self.fields.values():
            if isinstance(field, PermissionField) and self.instance.pk is not None:
                field.set_initial_value(self.permission_target)

    def save(self, commit=True):
        target = super().save(commit)
        to_remove = []
        to_assign = []
        for key, field in self.fields.items():
            if isinstance(field, PermissionField):
                if self.cleaned_data[key]:
                    to_assign.append(field)
                else:
                    to_remove.append(field)
        for field in to_remove:
            field.remove_permissions(target)
        for field in to_assign:
            field.assign_permissions(target)
        return target



from django.core.management import BaseCommand

from ephios.core.signals import periodic_signal


class Command(BaseCommand):
    def handle(self, *args, **options):
        periodic_signal.send(self)



from django.utils.translation import gettext_lazy as _

from ephios.core.plugins import PluginConfig


class PluginApp(PluginConfig):
    name = "ephios.plugins.qualification_management"

    class EphiosPluginMeta:
        name = _("Qualification Management")
        author = "Ephios Team"
        description = _("This plugins lets you add and edit qualifications.")

    def ready(self):
        from . import signals  # pylint: disable=unused-import



from django.core.management import BaseCommand

from ephios.core.services.notifications.backends import send_all_notifications


class Command(BaseCommand):
    def handle(self, *args, **options):
        send_all_notifications()



# Generated by Django 3.1.7 on 2021-03-24 20:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("core", "0006_notification"),
    ]

    operations = [
        migrations.CreateModel(
            name="EventAutoQualificationConfiguration",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "expiration_date",
                    models.DateField(blank=True, null=True, verbose_name="Expiration date"),
                ),
                (
                    "mode",
                    models.IntegerField(
                        choices=[(1, "any shift"), (2, "every shift"), (3, "last shift")],
                        default=1,
                        verbose_name="Which shifts must be attended to acquire the qualification?",
                    ),
                ),
                (
                    "extend_only",
                    models.BooleanField(
                        default=False,
                        verbose_name="Only extend or reactivate existing qualification",
                    ),
                ),
                (
                    "needs_confirmation",
                    models.BooleanField(
                        default=True, verbose_name="Qualification must be confirmed afterwards"
                    ),
                ),
                (
                    "event",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="auto_qualification_config",
                        to="core.event",
                    ),
                ),
                (
                    "qualification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="core.qualification"
                    ),
                ),
            ],
        ),
    ]



from crispy_forms.exceptions import CrispyError
from crispy_forms.utils import TEMPLATE_PACK
from django.forms import boundfield
from django.template import Context, Template
from django.template.loader import get_template


class AbortLink:
    """
    a-tag to cancel the form as a bootstrap button
    """

    def __init__(self, href):
        self.href = href

    def render(self, form, form_style, context, template_pack=TEMPLATE_PACK, **kwargs):
        context.update({"href": self.href})
        return Template(
            '{% load i18n %}<a role="button" class="btn btn-secondary mt-1" href="{{ href }}">{% trans "Cancel" %}</a>'
        ).render(context)


def as_crispy_field(
    field,
    template_pack=TEMPLATE_PACK,
    label_class="",
    field_class="",
    wrapper_class="",
    show_labels=True,
):
    # inspired by original as_crispy_field, but it is not callable as filter and does not support wrapper class/labels
    from django.conf import settings

    if not isinstance(field, boundfield.BoundField) and settings.DEBUG:
        raise CrispyError("|as_crispy_field got passed an invalid or inexistent field")

    attributes = {
        "field": field,
        "form_show_errors": True,
        "form_show_labels": show_labels,
        "label_class": label_class,
        "field_class": field_class,
        "wrapper_class": wrapper_class,
    }
    helper = getattr(field.form, "helper", None)

    template_path = None
    if helper is not None:
        attributes.update(helper.get_attributes(template_pack))
        template_path = helper.field_template
    if not template_path:
        template_path = f"{template_pack}/field.html"
    template = get_template(template_path)

    c = Context(attributes).flatten()
    return template.render(c)



from django.contrib import messages
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views import View
from django.views.generic.base import TemplateResponseMixin

from ephios.core.models import Event
from ephios.extra.mixins import CustomPermissionRequiredMixin


class EventBulkDeleteView(CustomPermissionRequiredMixin, TemplateResponseMixin, View):
    permission_required = "core.delete_event"
    template_name = "core/event_bulk_delete.html"

    def post(self, request, *args, **kwargs):
        events = Event.objects.filter(pk__in=request.POST.getlist("bulk_action"))
        if not events:
            messages.info(request, _("No events were selected for deletion."))
            return redirect(reverse("core:event_list"))
        if request.POST.get("confirm"):
            events.delete()
            messages.info(request, _("The selected events have been deleted."))
            return redirect(reverse("core:event_list"))
        return self.render_to_response({"events": events})



# Generated by Django 3.1.7 on 2021-04-03 21:48

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0007_auto_20210324_2101"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="consequence",
            options={"verbose_name": "Consequence"},
        ),
        migrations.AlterModelOptions(
            name="qualificationgrant",
            options={"verbose_name": "Qualification grant"},
        ),
        migrations.RemoveField(
            model_name="consequence",
            name="decided_by",
        ),
        migrations.RemoveField(
            model_name="consequence",
            name="executed_at",
        ),
        migrations.RemoveField(
            model_name="consequence",
            name="fail_reason",
        ),
        migrations.AlterField(
            model_name="abstractparticipation",
            name="finished",
            field=models.BooleanField(default=False, verbose_name="finished"),
        ),
        migrations.AlterField(
            model_name="consequence",
            name="state",
            field=models.TextField(
                choices=[
                    ("needs_confirmation", "needs confirmation"),
                    ("executed", "executed"),
                    ("failed", "failed"),
                    ("denied", "denied"),
                ],
                default="needs_confirmation",
                max_length=31,
                verbose_name="State",
            ),
        ),
        migrations.AlterField(
            model_name="localparticipation",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Participant",
            ),
        ),
        migrations.AlterField(
            model_name="shift",
            name="event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="shifts",
                to="core.event",
                verbose_name="event",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="is_active",
            field=models.BooleanField(default=True, verbose_name="Active"),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="is_staff",
            field=models.BooleanField(default=False, verbose_name="Staff user"),
        ),
    ]



from django.dispatch import receiver
from django.urls import reverse
from django.utils.translation import gettext as _

from ephios.core.signals import administration_settings_section, footer_link
from ephios.plugins.pages.models import Page


@receiver(footer_link, dispatch_uid="ephios.plugins.pages.signals.pages_footer_links")
def pages_footer_links(sender, request, **kwargs):
    pages = Page.objects.filter(show_in_footer=True)
    if request.user.is_anonymous:
        pages = pages.filter(publicly_visible=True)
    return {page.title: reverse("pages:page_detail", kwargs=dict(slug=page.slug)) for page in pages}


@receiver(
    administration_settings_section,
    dispatch_uid="ephios.plugins.pages.signals.pages_settings_section",
)
def pages_settings_section(sender, request, **kwargs):
    return [
        {
            "label": _("Pages"),
            "url": reverse("pages:settings_page_list"),
            "active": request.resolver_match.url_name.startswith("settings_page"),
        },
    ]
